<?php
if( !class_exists('SMSRU') ) {
  require_once( 'cf7smsru/class-SMSRU.php' );
}

CF7SMSRU_Plugin::get_smsru_auth();

abstract class CF7SMSRU_Plugin {
  
  public $plugin_callback = null;
  
  public $plugin_dir = null;
  
  protected $cf7smsru = null;

  public function __construct() {
    register_activation_hook( __FILE__, array( $this, 'install' ) );
    
    add_action( 'admin_head', array( $this, 'setup_admin_head' ) );  
    add_action( 'admin_menu', array( $this, 'setup_admin_navigation' ) );
    add_action( 'admin_notices', array( $this, 'setup_admin_message' ) ); 
    //add_action( 'admin_bar_menu', array( $this, 'setup_admin_bar' ), 999 );
    add_action( 'admin_init', array( $this, 'setup_admin_init' ) );
    add_action( 'admin_enqueue_scripts', array( $this, 'setup_cf7smsru_js' ) );
    
    $this->plugin_callback = array( $this, 'main' );
  }
    
     
  
  public function install() {
  }
  
  public function setup_admin_message() {
    $options = get_option( 'cf7smsru_options' );
	if(empty($options)) {
      $this->show_admin_message('Перед использованием плагина укажите API_id на <a href="' . site_url() . '/wp-admin/admin.php?page=cf7smsru_options">странице с настройками</a>. Если нет аккаунта, его необходимо <a href="http://gosend.sms.ru" target="_blank">создать</a>.');		
	} else {
		if( $options['api_id'] == '' ) {
		  $this->show_admin_message('Перед использованием плагина укажите API_id на <a href="' . site_url() . '/wp-admin/admin.php?page=cf7smsru_options">странице с настройками</a>. Если нет аккаунта, его необходимо <a href="http://gosend.sms.ru" target="_blank">создать</a>.');
		}
	}
  }
  
  
  public function setup_admin_navigation() {
    global $menu;
    
    $menu_exists = false;
    foreach( $menu as $k => $item ) {
      if( $item[0] == "CF7 SMS.ru" ) {
        $menu_exists = true;
        break;
      }
    }

    if( !$menu_exists ) {    
      add_menu_page( __( 'CF7 SMS.ru', $this->language_string ), __( 'CF7 SMS.ru', $this->language_string ), 'manage_options', 'cf7smsru_options', array( $this, 'cf7smsru_options' ) );
      add_submenu_page( 'cf7smsru_options', __( 'CF7 SMS.ru настройки', $this->language_string ), __( 'CF7 SMS.ru настройки', $this->language_string ), 'manage_options', 'cf7smsru_options', array( $this, 'cf7smsru_options' ) );
    }
    
    add_submenu_page( 'cf7smsru_options', __( $this->plugin_name, $this->language_string ), __( $this->plugin_name, $this->language_string ), 'manage_options', $this->plugin_callback[1], $this->plugin_callback );
  }
  
  public function setup_cf7smsru_js() {
		wp_enqueue_script( 'cf7smsru_options', plugins_url( 'js/cf7smsru_options.js', dirname( __FILE__ ) ), array( 'jquery' ) );
  }
  
  public function setup_admin_init() {
    register_setting( 'cf7smsru_options', 'cf7smsru_options', array( $this, 'cf7smsru_options_validate' ) );
    add_settings_section( 'cf7smsru_api_keys', 'Данные с сервиса SMS.ru(<a href="http://gosend.sms.ru" target="_blank">регистрация в SMS.ru</a>)', array( $this, 'settings_api_key_text' ), 'cf7smsru' );
	
	if (in_array('contact-form-7/wp-contact-form-7.php',apply_filters('active_plugins',get_option('active_plugins')))) {
		//add_settings_field( 'cf7smsru_api_key', 'Логин в системе', array( $this, 'settings_api_key_input' ), 'cf7smsru', 'cf7smsru_api_keys' );  
		add_settings_field( 'cf7smsru_api_id', 'Ваш api_id', array( $this, 'settings_api_id_input' ), 'cf7smsru', 'cf7smsru_api_keys' ); 
		add_settings_field( 'cf7smsru_name', 'Имя отправителя', array( $this, 'settings_name_input' ), 'cf7smsru', 'cf7smsru_api_keys' );   
		
		add_settings_section( 'cf7smsru_defaults', 'Ваш баланс', array( $this, 'settings_balans_text' ), 'cf7smsru' );  
		add_settings_section( 'cf7smsru_defaults1', 'Дневной лимит', array( $this, 'settings_limit_text' ), 'cf7smsru' ); 
		add_settings_section( 'cf7smsru_defaults2', 'Отправители', array( $this, 'settings_senders_text' ), 'cf7smsru' );  
	} else {
		add_settings_section( 'cf7smsru_defaults3', 'Установите плагин Contact Form 7', null, 'cf7smsru' );  		
	}	
  }
  
  public function settings_api_key_text() {
		echo '';
	}
  
  public function settings_balans_text() {
		echo $this->get_smsru_balance();
	}
  public function settings_limit_text() {
		echo $this->get_smsru_limit();
	}
  public function settings_senders_text() {
		echo $this->get_smsru_mysender();
	}
  
function get_smsru_mysender() {

	global $sms_wp, $log_sms_status;

	if ($log_sms_status) {
		$senders = $sms_wp->my_senders();
	
		foreach ($senders['senders'] as $key => $value) {
	        	echo 'Отправитель № ' . ($key + 1) . ' - ' . $value . '<br />';
	    	} 
		echo 'Для редактирования отправителей перейдите по сылке: <a href="https://gosend.sms.ru/?panel=senders">редактировать отправителей</a>';
	} else {
		echo 'Для получения информации по отправителям введите корректные данные в поля ниже.';
	}
}
function get_smsru_limit() {

	global $sms_wp, $log_sms_status;

	if ($log_sms_status) {
		$limit = $sms_wp->my_limit();
	
		echo 'Текущий лимит позволяет вам отправлять любое количество сообщений на ' . $limit['total'] . ' разных номеров в течение дня.<br />';
		echo 'Количество сообщений, отправленных вами в текущий день: ' . $limit['current'] . '<br />';
		echo 'Для увеличения лимита перейдите по сылке: <a href="http://gosend.sms.ru/?panel=support&subpanel=limit">увеличить лимит</a>';
	} else {
		echo 'Для получения информации по лимитам введите корректные данные в поля ниже.';
	}
}

public static function get_smsru_auth() {

	global $sms_wp, $log_sms_status;
	
    $options = get_option( 'cf7smsru_options' );
	
	$sms_wp = new SMSRU( $options['api_id'] );

		$login_status = $sms_wp->auth_check();
	
		if ($login_status['code'] == 100) {
			$log_sms_status = true;
		}

}
  
public function get_smsru_balance() {

	global $sms_wp, $log_sms_status;


	if ($log_sms_status) {
		$auth_status = $sms_wp->auth_check();
		if ($auth_status['code'] == 100) {
			$balance = $sms_wp->my_balance();
			echo $balance['balance'] . ' руб.';
		} else {
			$balance_status = $sms_wp->my_balance();
			echo $balance_status['description'];
		}
	} else {
		echo 'Для получения информации о балансе введите корректные данные в поля ниже.';
	}
}

  public function settings_api_key_input() {
    $options = get_option( 'cf7smsru_options' );
    
    if( isset( $options['api_key'] ) ) {         
        echo "<input id='cf7smsru_api_key' name='cf7smsru_options[api_key]' size='40' type='text' value='{$options['api_key']}' />";      
      return;
    } else {
      echo "<input id='cf7smsru_api_key' name='cf7smsru_options[api_key]' size='40' type='text' value='' />";           
    }
  }
  
  public function settings_api_id_input() {
    $options = get_option( 'cf7smsru_options' );
    
    if( isset( $options['api_id'] ) ) {      
      
        echo "<input id='cf7smsru_api_id' name='cf7smsru_options[api_id]' size='40' type='text' value='{$options['api_id']}' />";
      
      return;
    } else {
      echo "<input id='cf7smsru_api_id' name='cf7smsru_options[api_id]' size='40' type='text' value='' />";           
    }
  }
  
  public function settings_name_input() {
    $options = get_option( 'cf7smsru_options' );
    
    if( isset( $options['name'] ) ) {      
      
        echo "<input id='cf7smsru_name' name='cf7smsru_options[name]' size='40' type='text' value='{$options['name']}' /><br />Указанное имя должно быть согласовано с администрацией сервиса, иначе сообщения отправляться не будут, при пустом поле в имя отправителя подставляется Ваш номер.";
      
      return;
    } else {
      echo "<input id='cf7smsru_name' name='cf7smsru_options[name]' size='40' type='text' value='' /><br />Указанное имя должно быть согласовано с администрацией сервиса, иначе сообщения отправляться не будут, при пустом поле в имя отправителя подставляется Ваш номер.";           
    }
  }
  
  
  public function cf7smsru_options_validate( $val ) {
    // From santization
    $val['from'] = preg_replace( '/[^A-Za-z0-9]/', '', $val['from'] );
    if( preg_match( '/[0-9]/', $val['from'] ) ) {
      $val['from'] = substr( $val['from'], 0, 14 );
    } else {
      $val['from'] = substr( $val['from'], 0, 11 );
    }
   
        $key = '';
        add_settings_error( 'cf7smsru_options', 'cf7smsru_options', 'Ваши настройки сохраненны! Теперь вы можете пользоваться плагином.', 'updated' );
        return $val;
  }
  
  public function cf7smsru_options() {
    $this->render_template( 'cf7smsru-options' );
  }
  
  protected function show_admin_message( $message, $errormsg = false ) {
    if( $errormsg ) {
      echo '<div id="message" class="error">';
    } else {
      echo '<div id="message" class="updated fade">';
    }
  
    echo "<p><strong>$message</strong></p></div>";
  }
  
  protected function render_template( $name, $data = array() ) {
    include( WP_PLUGIN_DIR . '/' . $this->plugin_dir . '/templates/' . $name . '.php');
  }

}
